from zope.component import getUtility
from zope.publisher.browser import BrowserView
from plone.portlets.interfaces import IPortletManager
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName

class SubsiteView(BrowserView):

    @memoize
    def empty(self):
        dashboards = [getUtility(IPortletManager, name=name) for name in
                        ['ftw.subsite.front1',
                         'ftw.subsite.front2',
                         'ftw.subsite.front3',
                         'ftw.subsite.front4',
                         'ftw.subsite.front5',
                         'ftw.subsite.front6',
                         'ftw.subsite.front7']]

        num_portlets = 0
        for dashboard in dashboards:
            num_portlets += len(dashboard)
        return num_portlets == 0

    def is_anon(self):
        member_tool = getToolByName(self, 'portal_membership')
        isAnon = member_tool.isAnonymousUser()
        return isAnon



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import plone
from unittest import TestCase


class TestSubsite(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    @browsing
    def test_subsite_addable(self, browser):
        browser.login().visit()
        factoriesmenu.add('Subsite')
        browser.fill({'Title': 'This is a Subsite'}).save()

        self.assertEquals('This is a Subsite', plone.first_heading())

    @browsing
    def test_subsite_has_simplelayout_as_default_page(self, browser):
        subsite = create(Builder('subsite').titled(u'A Subsite'))
        browser.login().visit(subsite)
        self.assertTrue(browser.css('.sl-simplelayout'))



from plone.app.layout.viewlets.common import ViewletBase
from plone.app.layout.navigation.root import getNavigationRoot
from ftw.subsite.interfaces import ISubsite
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class CSSViewlet(ViewletBase):

    template = ViewPageTemplateFile('cssviewlet.pt')

    def __init__(self, context, request, view, manager=None):
        super(CSSViewlet, self).__init__(context, request, view, manager)
        self.nav_root = None

    def render(self):
        self.nav_root = self.context.restrictedTraverse(
            getNavigationRoot(self.context), None)
        if ISubsite.providedBy(self.nav_root):
            return self.template()
        else:
            return ''

    def get_css(self):
        return self.nav_root.additional_css



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from unittest import TestCase
import os

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
IMAGE_PATH = os.path.join(CURRENT_DIR, 'assets/4dreamwork.svg')


class TestFtwLogoIntegration(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    @browsing
    def test_logo_edit_view_exists_on_site_root(self, browser):
        browser.login()
        # ftw-logo-overrides/edit
        browser.open(self.portal, view='logo-and-icon-overrides')
        # I assert the title because I assume that if the title
        # is loaded we successfully loaded the page
        self.assertEqual('Manually set site Logos and Icons',
                         browser.css('h1').first.text,
                         'The pages title should be different')

    @browsing
    def test_logo_on_site_root_comming_from_ftw_logo_source(self, browser):
        browser.login()
        page = create(Builder('sl content page').titled(u'Hugo'))
        browser.open(page)
        logo_link = browser.css('#portal-logo img').first.node.attrib.get('src')
        logo_link_without_query = logo_link.split('?')[0]
        self.assertEqual('http://nohost/plone/@@logo/logo/get_logo',
                         logo_link_without_query,
                         'The logo should be implemented from ftw.logo')

    @browsing
    def test_logo_can_be_changed_for_plone_root(self, browser):
        browser.login()
        upload_context = self.portal
        logos = self.upload_image_and_return_actual_and_expected_image(
            browser, upload_context)

        self.assertEqual(logos['expected'],
                         logos['actual'],
                         'We expect that the logo on the page is the same as '
                         'the one we\'ve uploaded with ftw.logo')

    @browsing
    def test_logo_can_be_changed_for_subsite(self, browser):
        page = create(Builder('sl content page').titled(u'Wrapper Page'))
        subsite = create(Builder('subsite').titled(u'Subsite').within(page))

        browser.login()
        upload_context = subsite
        logos = self.upload_image_and_return_actual_and_expected_image(
            browser, upload_context)

        self.assertEqual(logos['expected'],
                         logos['actual'],
                         'We expect that the logo on the page is the same as '
                         'the one we\'ve uploaded with ftw.logo')

    @browsing
    def test_subsite_logo_change_does_not_affect_other_sl_page(self, browser):
        page_one = create(Builder('sl content page').titled(u'Wrapper Page 1'))
        subsite = create(Builder('subsite').titled(u'Subsite').within(page_one))
        page_two = create(Builder('sl content page').titled(u'Wrapper Page 2'))

        browser.login()

        logos_changed = self.upload_image_and_return_actual_and_expected_image(
            browser, subsite)
        self.assertEqual(logos_changed['expected'],
                         logos_changed['actual'],
                         'We expect that the logo on the first page is the '
                         'same as the one we\'ve uploaded with ftw.logo')

        logos_unchanged = self.return_actual_and_expected_image_but_do_not_upload(
            browser, page_two)
        self.assertNotEqual(logos_unchanged['expected'],
                            logos_unchanged['actual'],
                            'We expect that the second pages logo is not the '
                            'the one we\'ve uploaded on page one.')

    @browsing
    def test_subsite_logo_change_does_not_affect_other_subsite(self, browser):
        page_one = create(Builder('sl content page').titled(u'Wrapper Page 1'))
        subsite = create(Builder('subsite').titled(u'Subsite').within(page_one))
        page_two = create(Builder('sl content page').titled(u'Wrapper Page 2'))
        subsite_two = create(Builder(
            'subsite').titled(u'Subsite').within(page_two))

        browser.login()

        logos_changed = self.upload_image_and_return_actual_and_expected_image(
            browser, subsite)
        self.assertEqual(logos_changed['expected'],
                         logos_changed['actual'],
                         'We expect that the logo on the first subsite is the '
                         'same as the one we\'ve uploaded with ftw.logo')

        logos_unchanged = self.return_actual_and_expected_image_but_do_not_upload(
            browser, subsite_two)
        self.assertNotEqual(logos_unchanged['expected'],
                            logos_unchanged['actual'],
                            'We expect that the second subsites logo is not '
                            'the same as the one we\'ve uploaded on first.')

    @browsing
    def test_subsite_logo_change_changes_sl_pages_logo_in_subsite(self, browser):
        wrapper_page = create(Builder('sl content page').titled(u'Wrapper Page 2'))
        subsite = create(Builder(
            'subsite').titled(u'Subsite').within(wrapper_page))
        page = create(Builder('sl content page').titled(
            u'Test Page').within(subsite))

        browser.login()

        # upload new base logo
        browser.open(subsite, view='logo-and-icon-overrides')
        with open(IMAGE_PATH) as file_:
            expected_logo = file_.read()
            file_.seek(0)
            browser.fill({'form.widgets.logo_BASE': file_}).submit()

        # open actual image to compare with the expected
        browser.open(page)
        logo_link = browser.css(
            '#portal-logo img').first.node.attrib.get('src')
        logo_link_without_query = logo_link.split('?')[0]
        browser.open(logo_link_without_query)
        actual_logo = browser.contents

        self.assertEqual(expected_logo,
                         actual_logo,
                         'We expect, that the logo of a page within a subsite '
                         'of which we changed the logo has the subsites logo.')

    @staticmethod
    def upload_image_and_return_actual_and_expected_image(
            browser, upload_context):

        # upload new base logo
        browser.open(upload_context, view='logo-and-icon-overrides')
        with open(IMAGE_PATH) as file_:
            expected_logo = file_.read()
            file_.seek(0)
            browser.fill({'form.widgets.logo_BASE': file_}).submit()

        # open actual image to compare with the expected
        browser.open(upload_context)
        logo_link = browser.css(
            '#portal-logo img').first.node.attrib.get('src')
        logo_link_without_query = logo_link.split('?')[0]
        browser.open(logo_link_without_query)
        actual_logo = browser.contents

        return {'expected': expected_logo, 'actual': actual_logo}

    @staticmethod
    def return_actual_and_expected_image_but_do_not_upload(
            browser, context):

        with open(IMAGE_PATH) as file_:
            expected_logo = file_.read()

        # open actual image to compare with the expected
        browser.open(context)
        logo_link = browser.css(
            '#portal-logo img').first.node.attrib.get('src')
        logo_link_without_query = logo_link.split('?')[0]
        browser.open(logo_link_without_query)
        actual_logo = browser.contents

        return {'expected': expected_logo, 'actual': actual_logo}



from ftw.upgrade import UpgradeStep
from ftw.upgrade.migration import BACKUP_AND_IGNORE_UNMAPPED_FIELDS
from ftw.upgrade.migration import InplaceMigrator


class MigrateToDexterity(UpgradeStep):
    """Migrate to dexterity.
    """

    def __call__(self):
        self.setup_install_profile('profile-ftw.simplelayout.contenttypes:default')
        self.install_upgrade_profile()

        migrator = InplaceMigrator(
            new_portal_type='ftw.subsite.Subsite',
            options=BACKUP_AND_IGNORE_UNMAPPED_FIELDS,
            field_mapping={
                'FromEmail': 'from_email',
                'FromName': 'from_name',
                'forcelanguage': 'force_language',
                'linkSiteInLanguagechooser': 'link_site_in_languagechooser',
            }
        )

        map(migrator.migrate_object,
            self.objects({'portal_type': 'Subsite'}, 'Migrate to dexterity.'))



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.subsite.testing import FTW_SUBSITE_SPECIAL_FUNCTIONAL_TESTING
from ftw.subsite.tests.helpers import LanguageSetter
from ftw.testbrowser import browsing
from ftw.testing import IS_PLONE_5
from unittest import TestCase
from zope.i18n.locales import locales

CSS_SELECTOR_SITE_ACTIONS = '.actions-site_actions' if IS_PLONE_5 else '#portal-siteactions'


class TestSubsiteForceLanguage(TestCase, LanguageSetter):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        default = 'en'
        supported = ['en', 'de']
        self.set_language_settings(default, supported)

    def _set_language_de(self):
        """This Function is used to set the language of the plone site.
        We need this, because we wan't to make sure that the language is
        inherited when there isn't one forced.
        """
        locale = locales.getLocale('de')
        target_language = locale.id.language

        # If we get a territory, we enable the combined language codes
        use_combined = False
        if locale.id.territory:
            use_combined = True
            target_language += '_' + locale.id.territory

        # As we have a sensible language code set now, we disable the
        # start neutral functionality (not available in plone 5.1 anymore).
        start_neutral = False

        self.set_language_settings(target_language, [target_language],
                                   use_combined, start_neutral)

    @browsing
    def test_language_plone_root(self, browser):
        browser.login().visit()
        # Grap some translated shizzle on the plone root site -
        # like site actions
        self.assertEquals(browser.css(CSS_SELECTOR_SITE_ACTIONS).text,
                          ['Site Map Accessibility Contact'])

    @browsing
    def test_inherit_language_from_plone_root(self, browser):
        subsite = create(Builder('subsite').titled(u'A Subsite'))
        browser.login().visit(subsite)
        self.assertEquals(['Site Map Accessibility Contact'],
                          browser.css(CSS_SELECTOR_SITE_ACTIONS).text)

    @browsing
    def test_change_language_to_de_on_subsite(self, browser):
        subsite = create(Builder('subsite')
                         .titled(u'Subsite')
                         .with_language('de'))
        self.set_language_settings()
        browser.login().visit(subsite)

        self.assertEquals([u'\xdcbersicht Barrierefreiheit Kontakt'],
                          browser.css(CSS_SELECTOR_SITE_ACTIONS).text,)

    @browsing
    def test_language_changed_to_de_also_on_subsite_subcontent(self, browser):
        subsite = create(Builder('subsite')
                         .titled(u'Subsite')
                         .with_language('de'))
        folder = create(Builder('folder').titled(u'A Folder').within(subsite))
        self.set_language_settings()

        browser.login().visit(folder)
        self.assertEquals([u'\xdcbersicht Barrierefreiheit Kontakt'],
                          browser.css(CSS_SELECTOR_SITE_ACTIONS).text)

    @browsing
    def test_language_inherited_on_subcontent_of_subsite(self, browser):
        self._set_language_de()
        subsite = create(Builder('subsite').titled(u'Subsite'))
        folder = create(Builder('folder').within(subsite))
        browser.login().visit(folder)
        self.assertEquals([u'\xdcbersicht Barrierefreiheit Kontakt'],
                          browser.css(CSS_SELECTOR_SITE_ACTIONS).text)


class TestNegotiatorSpecialCase(TestCase):

    # Do not setup ftw.subsite to test if the customized negotiator behaves
    # normal
    layer = FTW_SUBSITE_SPECIAL_FUNCTIONAL_TESTING

    @browsing
    def test_subsitelayer_not_available(self, browser):
        # The Subsite browserlayer is not available, so it should behave like
        # Plone default
        folder = create(Builder('folder').titled(u'Subsite'))
        browser.login().visit(folder)
        self.assertEquals(['Site Map Accessibility Contact'],
                          browser.css(CSS_SELECTOR_SITE_ACTIONS).text)



from ftw.subsite.interfaces import ILanguages
from plone.app.layout.viewlets import common
from zope.component import getMultiAdapter
from zope.interface import implements
from zope.viewlet.interfaces import IViewlet
from zope.publisher.browser import BrowserView


class Languages(BrowserView):

    def available(self):
        langs = getMultiAdapter((self.context, self.request), ILanguages)
        return bool(langs.get_related_languages())

    def current(self):
        langs = getMultiAdapter((self.context, self.request), ILanguages)
        return langs.get_current_language()


    def languages(self):
        langs = getMultiAdapter((self.context, self.request), ILanguages)
        return langs.get_related_languages()


class LanguageSelector(common.ViewletBase, Languages):
    implements(IViewlet)



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.browser.sitemap import SubsiteSitemapQueryBuilder
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from unittest import TestCase


class TestSubsiteSitemap(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        self.subsite = create(Builder('subsite').titled(u'My Subsite'))
        self.folder = create(Builder('folder')
                             .titled(u'Folder')
                             .within(self.subsite))

    def test_subsite_query_builder_on_plone_root(self):
        query = SubsiteSitemapQueryBuilder(self.portal)()

        self.assertEquals(query['path']['query'], '/plone',
                          'Expect portal root')

    def test_subsite_query_builder_on_subsite(self):
        query = SubsiteSitemapQueryBuilder(self.subsite)()

        self.assertEquals(query['path']['query'], '/plone/my-subsite',
                          'Expect subsite path')

    def test_subsite_query_builder_on_folder_in_subsite(self):
        query = SubsiteSitemapQueryBuilder(self.folder)()

        self.assertEquals(
            query['path']['query'], '/plone/my-subsite',
            'Expect subsite path, because the folder is in a subsite')

    @browsing
    def test_subsite_sitemap_view_on_plone_root(self, browser):
        browser.login().visit(self.portal, view='sitemap')
        self.assertEquals(['My Subsite', 'Folder'],
                          browser.css('#portal-sitemap a').text)

    @browsing
    def test_subsite_sitemap_view_on_subsite_root(self, browser):
        browser.login().visit(self.portal, view='sitemap')
        self.assertEquals(['My Subsite', 'Folder'],
                          browser.css('#portal-sitemap a').text)



from ftw.testing import IS_PLONE_5
from plone import api
from plone.registry.interfaces import IRegistry
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
import transaction


def introduce_language_subsites(*subsites):
    intids = getUtility(IIntIds)

    for subsite in subsites:
        ids = [intids.getId(obj) for obj in subsites]
        ids.remove(intids.getId(subsite))
        subsite.language_references = [RelationValue(id_) for id_ in ids]

        transaction.commit()


class LanguageSetter(object):

    def set_language_settings(self, default='en', supported=None,
                              use_combined=False, start_neutral=True):
        """
        Sets language settings regardeless if plone4.3 or plone5.1
        :param default: default site language
        :param supported: list of supported languages
        """
        # startNeutral is not used/available in plone 5.1 anymore

        if not supported:
            supported = ['en']

        if IS_PLONE_5:
            from Products.CMFPlone.interfaces import ILanguageSchema

            self.ltool = api.portal.get_tool('portal_languages')
            self.ltool.setDefaultLanguage(default)
            for lang in supported:
                self.ltool.addSupportedLanguage(lang)
            self.ltool.settings.use_combined_language_codes = False
            self.ltool.setLanguageCookie()
            registry = getUtility(IRegistry)
            language_settings = registry.forInterface(ILanguageSchema, prefix='plone')
            language_settings.use_content_negotiation = True
        else:
            self.ltool = self.portal.portal_languages
            self.ltool.manage_setLanguageSettings(
                default,
                supported,
                setUseCombinedLanguageCodes=use_combined,
                # Set this only for better testing ability
                setCookieEverywhere=True,
                startNeutral=start_neutral,
                setContentN=True)
        transaction.commit()



from plone.app.layout.viewlets import common
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.memoize import instance
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.navigation.root import getNavigationRoot
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from random import choice
from zExceptions import Unauthorized
from Products.CMFCore.utils import getToolByName


class Banner(common.ViewletBase):
    render = ViewPageTemplateFile('banner_viewlet.pt')

    @property
    def available(self):
        if not self.get_banners():
            return False

        registry = getUtility(IRegistry)
        root_only = registry.get('ftw.subsite.banner_root_only', True)

        if root_only:
            context_state = self.context.restrictedTraverse(
                'plone_context_state')
            return context_state.is_view_template()

        return True

    @instance.memoize
    def get_banners(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        bannerfolder = self.get_banner_folder()
        if not bannerfolder:
            return []

        registry = getUtility(IRegistry)
        root_only = registry.get('ftw.subsite.banner_root_only', True)

        query = dict(portal_type='Image',
                     path='/'.join(bannerfolder.getPhysicalPath()))
        imgs = [brain.getObject() for brain in catalog(query)]

        if root_only:
            context_state = self.context.restrictedTraverse(
                '@@plone_context_state')
            nav_root = INavigationRoot.providedBy(self.context)
            if context_state.is_portal_root() or nav_root:
                return imgs
            return []
        else:
            return imgs

    def get_banner_folder(self):
        registry = getUtility(IRegistry)

        name = registry.get('ftw.subsite.bannerfoldername', 'banners')
        bannerfolder = None
        try:
            nav_context = self.context.restrictedTraverse(
                getNavigationRoot(self.context)).aq_explicit

            bannerfolder = nav_context.restrictedTraverse(name.encode('utf-8'))
        except (KeyError, AttributeError):
            return None
        except Unauthorized:
            return None
        return bannerfolder

    def get_banner_tag(self):
        title = self.context.title_or_id()
        img = choice(self.get_banners())
        scales = img.restrictedTraverse('@@images')
        return scales.tag('image', scale='bannerimage', alt=title,
                            title=title)



from ftw.upgrade import UpgradeStep
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.component import getUtility


class FixAttributes(UpgradeStep):

    MANAGER_NAMES = set("ftw.subsite.front{0}".format(index)
                        for index in range(1, 8))

    def __call__(self):
        self.migrate_portlet_assignments(self.portal)
        for obj in self.objects(
                {'object_provides':
                    'plone.app.layout.navigation.interfaces.INavigationRoot'},
                'Migrate teaser portlet assignments'):
            self.migrate_portlet_assignments(obj)

    def migrate_portlet_assignments(self, obj):
        from ftw.subsite.portlets.teaserportlet import ITeaserPortlet

        try:
            annotations = IAnnotations(obj)
        except TypeError:
            # The obj has no annotations
            return

        known_manager = annotations.get(
            'plone.portlets.contextassignments', [])

        for managername in tuple(known_manager):
            if managername not in self.MANAGER_NAMES:
                continue

            manager = getUtility(IPortletManager, name=managername,
                                 context=obj)
            assignments = getMultiAdapter((obj, manager),
                                          IPortletAssignmentMapping,
                                          context=obj)
            assignments._p_changed = True

            for assignment in assignments.values():
                if not ITeaserPortlet.providedBy(assignment):
                    continue

                if not assignment.__dict__.get('image'):
                    continue

                assignment.image = assignment.__dict__.pop('image', None)
                assignment._p_changed = True



from ftw.builder.dexterity import DexterityBuilder
from ftw.builder import builder_registry
from ftw.simplelayout.tests import builders


class SubsiteBuilder(DexterityBuilder):

    portal_type = 'ftw.subsite.Subsite'

    def with_language(self, language_code):
        if 'title' not in self.arguments:
            return self.having(force_language=language_code,
                               title=language_code)
        else:
            return self.having(force_language=language_code)


builder_registry.register('subsite', SubsiteBuilder)



from ftw.subsite.languages import translate_language
from ftw.subsite.testing import FTW_SUBSITE_INTEGRATION_TESTING
import unittest as unittest


class TestGetLanguageNative(unittest.TestCase):

    layer = FTW_SUBSITE_INTEGRATION_TESTING

    def test_return_native_for_existing_language(self):
        """Test If our function returns the correct Native, if the Language
        exists"""
        native = translate_language(self.layer['portal'], 'sv')
        self.assertEqual(u'Svenska', native)

    def test_returns_none_for_invalid_language(self):
        """Check if function returns nothing when the language
        doesn't exist.'"""
        native = translate_language(self.layer['portal'], 'hans')
        self.assertEqual(None, native)



from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from email.header import Header
from email.mime.text import MIMEText
from ftw.subsite import _
from plone.app.layout.navigation.root import getNavigationRoot
from plone.dexterity.utils import safe_utf8
from z3c.form import form, field, button
from z3c.form.validator import SimpleFieldValidator
from z3c.form.validator import WidgetValidatorDiscriminators
from zope import schema
from zope.component import provideAdapter
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import Invalid
import pkg_resources
import re


IS_PLONE_5 = pkg_resources.get_distribution('Products.CMFPlone').version >= '5'


class IContactForm(Interface):
    """Interface for z3c.form"""
    sender = schema.TextLine(title=_(u"Sender_Name", default=u"Name"),
                             required=True,
                             description=_(u"help_Sender", default="Please enter your Name"))
    email = schema.TextLine(title=_(u"mail_address", default="E-Mail"),
                            required=True,
                            description=_(u"help_mail_address", default="Please enter your e-mail"))
    subject = schema.TextLine(title=_(u"label_subject", default="Subject"),
                              required=True,
                              description=_(u"help_subject", default="Please enter a Subject"))
    message = schema.Text(title=_(u"label_message", default="Message"),
                          required=True,
                          description=_(u"help_message", default="Please enter a Message"))


class ContactForm(form.Form):
    label = _(u"label_send_feedback", default=u"Send Feedback")
    fields = field.Fields(IContactForm)
    # don't use context to get widget data
    ignoreContext = True

    @button.buttonAndHandler(_(u'Send Mail'))
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            return

        message = data.get('message')
        email = data.get('email')
        subject = data.get('subject')
        sender = data.get('sender')
        self.send_feedback(email, subject, message, sender)
        msg = _(u'info_email_sent', default=u'The email was sent.')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')
        return self.redirect()

    @button.buttonAndHandler(_(u'button_cancel', default=u'Cancel'))
    def handle_cancel(self, action):
        return self.redirect()

    def redirect(self):
        """Redirect back
        """
        url = self.context.absolute_url()
        return self.request.RESPONSE.redirect(url)

    def send_feedback(self, recipient, subject, message, sender):
        """Send a feedback email to the email address defined in subsite.
        """
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        nav_root = None
        if not '/'.join(portal.getPhysicalPath()) == getNavigationRoot(self.context):
            nav_root = self.context.restrictedTraverse(
                getNavigationRoot(self.context))
        if nav_root:
            site_title = nav_root.Title()
            site_url = nav_root.absolute_url()
        else:
            site_title = portal.Title()
            site_url = portal.absolute_url()

        message = translate(
            u'feedback_mail_text',
            domain='ftw.subsite',
            default='${sender} sends you a message from your site ${site_title} (${site_url}):\n${msg}',
            context=self.request,
            mapping={'sender': "%s (%s)" % (sender, recipient),
                     'msg': message,
                     'site_title': site_title,
                     'site_url': site_url})

        default_from_name = portal.getProperty('email_from_name', '')
        default_from_email = portal.getProperty('email_from_address', '')

        if nav_root:
            email_from_name = nav_root.from_name or default_from_name
            email_from_email = nav_root.from_email or default_from_email
        else:
            email_from_name = default_from_name
            email_from_email = default_from_email

        if IS_PLONE_5:
            from plone import api
            reg = api.portal.get_tool('portal_registry')
            from_email_field = reg._records.get('plone.email_from_address')
            if not email_from_email and not from_email_field.value:
                # especially for testing reasons we need to set a value here
                # if none is registered on the page in plone5
                from_email_field._set_value('site@nohost.com')
                email_from_email = from_email_field.value

            email_from_email = (email_from_email or
                                reg._records.get('plone.email_from_address').value)
            email_from_name = (email_from_name or
                               reg._records.get('plone.email_from_name').value)

        # send the message
        self.send_mail(email_from_name, email_from_email, email_from_email,
                       subject, message, recipient, sender)

    def send_mail(self, name_from, mail_from, mail_to, mail_subject,
                  message_text, reply_mail, reply_name):
        """Send mail using plone mailhost (all input strings need to be utf-8).
        """
        if not name_from:
            name_from = ''
        # Put together the mail parts
        msg = MIMEText(safe_utf8(message_text), 'plain', 'utf-8')
        msg['From'] = '{} <{}>'.format(safe_utf8(name_from),
                                       safe_utf8(mail_from))
        msg['To'] = safe_utf8(mail_to)
        msg['Subject'] = Header(safe_utf8(mail_subject), 'utf-8')
        msg['reply-to'] = '{} <{}>'.format(safe_utf8(reply_name),
                                           safe_utf8(reply_mail))

        # Get mailhost and send to mail_to
        mailhost = getToolByName(self.context, 'MailHost')
        mailhost.send(msg)


class AddressesValidator(SimpleFieldValidator):
    """Validator for validating the e-mail addresses field
    """

    MAIL_EXPRESSION = r"^(\w&.%#$&'\*+-/=?^_`{}|~]+!)*[\w&.%#$&'\*+-/=" +\
        "?^_`{}|~]+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?" +\
        "\.)+[a-z]{2,6}|([0-9]{1,3}\.){3}[0-9]{1,3})$"

    def __init__(self, *args, **kwargs):
        super(AddressesValidator, self).__init__(*args, **kwargs)
        self.email_expression = re.compile(AddressesValidator.MAIL_EXPRESSION,
                                           re.IGNORECASE)

    def validate(self, value):
        """Validates the `value`, expects a list of carriage-return-separated
        email addresses.
        """
        super(AddressesValidator, self).validate(value)
        if value:
            address = value.strip()
            self._validate_addresses(address)

    def _validate_addresses(self, address):
        """E-Mail address validation
        """
        address = address.strip()
        if not self.email_expression.match(address):
            msg = _(u'error_invalid_addresses',
                    default=u'Your E-mail address is not valid.')
            raise Invalid(msg)


WidgetValidatorDiscriminators(AddressesValidator,
                              field=IContactForm['email'])
provideAdapter(AddressesValidator)



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.interfaces import IFtwSubsiteLayer
from ftw.subsite.testing import FTW_SUBSITE_INTEGRATION_TESTING
from ftw.testing import IS_PLONE_5
from plone.app.testing import login
from plone.registry import Record, field
from plone.registry.interfaces import IRegistry
from unittest import TestCase
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.publisher.browser import BrowserView
from zope.viewlet.interfaces import IViewletManager
import os


class TestBannerViewlet(TestCase):

    layer = FTW_SUBSITE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def _setup_bannerfolder(self, context):
        registry = getUtility(IRegistry)
        bannerfoldername = registry.get('ftw.subsite.bannerfoldername')
        return create(Builder('folder')
                      .within(context)
                      .titled(bannerfoldername))

    def _get_viewlet(self, context, view=None):
        if not view:
            view = BrowserView(context, context.REQUEST)
            context_state = context.restrictedTraverse(
                'plone_context_state')
            view.__name__ = context_state.view_template_id()

        context.REQUEST['ACTUAL_URL'] = '/'.join((context.absolute_url(),
                                                  view.__name__))
        # invalidate caches
        del IAnnotations(context.REQUEST)['plone.memoize']

        alsoProvides(context.REQUEST, IFtwSubsiteLayer)
        manager = queryMultiAdapter(
            (context, context.REQUEST, view),
            IViewletManager,
            'plone.portalheader')
        self.failUnless(manager)
        # Set up viewlets
        manager.update()
        name = 'ftw.subsite.banner'
        return [v for v in manager.viewlets if v.__name__ == name]

    def test_component_registered(self):
        self.assertTrue(len(self._get_viewlet(self.portal)) == 1)

    def test_get_banner_folder(self):
        viewlet = self._get_viewlet(self.portal)

        # Currently there's no bannerfolder
        self.assertIsNone(viewlet[0].get_banner_folder())

        # Add banner folder
        bannerfolder = self._setup_bannerfolder(self.portal)
        self.assertEquals(viewlet[0].get_banner_folder(), bannerfolder)

    def test_get_banners(self):
        viewlet = self._get_viewlet(self.portal)
        # No banner folder
        self.assertEquals(viewlet[0].get_banners(), [])
        self.assertFalse(viewlet[0].available)

        # Add banner folder
        bannerfolder = self._setup_bannerfolder(self.portal)

        # Bannerfolder is empty
        self.assertEquals(viewlet[0].get_banners(), [])
        self.assertFalse(viewlet[0].available)

        # Add images
        image1 = bannerfolder.get(bannerfolder.invokeFactory('Image',
                                                             'image1'))
        image2 = bannerfolder.get(bannerfolder.invokeFactory('Image',
                                                             'image2'))
        if IS_PLONE_5:
            image1.reindexObject()
            image2.reindexObject()
        else:
            image1.processForm()
            image2.processForm()
        viewlet = self._get_viewlet(self.portal)  # It's cached
        self.assertEquals(len(viewlet[0].get_banners()), 2)
        self.assertTrue(viewlet[0].available)

    def test_get_banner_tag(self):
        image = self.help_import_and_preprocess_image_for_AT_and_Dext()
        viewlet = self._get_viewlet(self.portal)
        self.assertIn("height", viewlet[0].get_banner_tag())
        self.assertIn("width", viewlet[0].get_banner_tag())
        self.assertIn(self.portal.title_or_id(), viewlet[0].get_banner_tag())
        self.assertIn("%s/@@images" % image.absolute_url(),
                      viewlet[0].get_banner_tag())

    def test_available_on_navroot(self):
        self.help_import_and_preprocess_image_for_AT_and_Dext()
        viewlet = self._get_viewlet(self.portal)
        # Default is only on navroot
        self.assertTrue(viewlet[0].available)

        folder = self.portal.get(self.portal.invokeFactory('Folder',
                                                           'folder'))
        # Not on sub content
        viewlet = self._get_viewlet(folder)
        self.assertFalse(viewlet[0].available)

    def test_available_everywhere(self):
        self.help_import_and_preprocess_image_for_AT_and_Dext()
        registry = getUtility(IRegistry)
        registry.records['ftw.subsite.banner_root_only'] = \
            Record(field.Bool(title=u"dummy", default=True),
                   value=False)

        viewlet = self._get_viewlet(self.portal)
        # On nav root
        self.assertTrue(viewlet[0].available)

        folder = self.portal.get(self.portal.invokeFactory('Folder',
                                                           'folder'))
        # On sub content
        viewlet = self._get_viewlet(folder)
        self.assertTrue(viewlet[0].available)

    def test_on_subsite(self):
        subsite = create(Builder('subsite').titled(u'MySubsite'))

        viewlet = self._get_viewlet(subsite)
        self.assertTrue(len(viewlet) == 1)
        image = self.help_import_and_preprocess_image_for_AT_and_Dext(subsite)

        self.assertIn("MySubsite", viewlet[0].get_banner_tag())
        self.assertIn("%s/@@images" % image.absolute_url(),
                      viewlet[0].get_banner_tag())

    def test_find_image_in_subfolder(self):
        bannerfolder = self._setup_bannerfolder(self.portal)
        subfolder = create(Builder('folder')
                           .titled(u'Subfolder')
                           .within(bannerfolder))
        create(Builder('image').within(subfolder))

        viewlet = self._get_viewlet(self.portal)
        self.assertTrue(viewlet[0].available, 'Expect to find an image')

    def test_do_not_show_on_folder_contents_when_root_only_enabled(self):
        registry = getUtility(IRegistry)
        registry.records['ftw.subsite.banner_root_only'] = \
            Record(field.Bool(title=u"dummy", default=True),
                   value=True)

        bannerfolder = self._setup_bannerfolder(self.portal)
        subfolder = bannerfolder.get(
            bannerfolder.invokeFactory('Folder', 'subfolder'))
        subfolder.invokeFactory('Image', 'image1')

        viewlet = self._get_viewlet(self.portal)
        self.assertTrue(
            viewlet[0].available,
            'Expected viewlet to be available on default view')

        folder_contents = self.portal.restrictedTraverse('folder_contents')
        viewlet = self._get_viewlet(self.portal, view=folder_contents)
        self.assertFalse(
            viewlet[0].available,
            'Expected viewlet not to be available on folder_contents,'
            ' since root_only is enabled.')

    def test_show_on_folder_contents_when_root_only_disabled(self):
        registry = getUtility(IRegistry)
        registry.records['ftw.subsite.banner_root_only'] = \
            Record(field.Bool(title=u"dummy", default=True),
                   value=False)

        bannerfolder = self._setup_bannerfolder(self.portal)
        subfolder = bannerfolder.get(
            bannerfolder.invokeFactory('Folder', 'subfolder'))
        subfolder.invokeFactory('Image', 'image1')

        viewlet = self._get_viewlet(self.portal)
        self.assertTrue(
            viewlet[0].available,
            'Expected viewlet to be available on default view')

        folder_contents = self.portal.restrictedTraverse('folder_contents')
        viewlet = self._get_viewlet(self.portal, view=folder_contents)
        self.assertTrue(
            viewlet[0].available,
            'Expected viewlet to be available on folder_contents,'
            ' since root_only is disabled.')

    def test_do_not_fail_if_nav_root_is_not_traversable(self):
        self.portal.portal_workflow.setChainForPortalTypes(
            ('Folder', 'ftw.subsite.Subsite'),
            'simple_publication_workflow')

        self._setup_bannerfolder(self.portal)
        subsite = create(Builder('subsite').titled(u'MySubsite'))
        subfolder = create(Builder('folder').within(subsite))
        user = create(Builder('user').with_roles('Manager', on=subfolder))

        login(self.portal, user.getId())
        self.assertFalse(self._get_viewlet(subfolder)[0].available)

    def help_import_and_preprocess_image_for_AT_and_Dext(self, add_withing=None):
        if not add_withing:
            add_withing = self.portal
        bannerfolder = self._setup_bannerfolder(add_withing)
        file_ = open("%s/blue.png" % os.path.split(__file__)[0], 'r')
        if IS_PLONE_5:
            from plone.namedfile.file import NamedImage
            file_ = NamedImage(data=file_, filename=u'blue.png')
            image = bannerfolder.get(
                bannerfolder.invokeFactory('Image', 'image1', image=file_))
            image.reindexObject()
        else:
            image = bannerfolder.get(
                bannerfolder.invokeFactory('Image', 'image1', image=file_))
            image.processForm()

        return image



from zope.interface import Interface
from plone.app.portlets.interfaces import IColumn


class ISubsite(Interface):
    """
    marker interface for subsite content type
    """


class ISubsiteColumn(IColumn):
    """
    Marker Inerface for front view columns
    """


class IFtwSubsiteLayer(Interface):
    """
    Browserlayer
    """

class ILanguages(Interface):
    """Adapter for handling language specific questions for a context.
    It respects interaction between a certain set of connected subsites
    with different translations as well as for subsites connected to the
    site root regarding languages.

    The adapter can be used on any context and finds the next parent subsite
    or plone site root which dictactes the settings for the contents below.
    """

    def __init__(context, request):
        """ILanguages is a multi adapter adapting context and request.
        """

    def get_current_language(self):
        """Returns the language active in this area as dict.
        Example:

        >>> {'url': 'http://url/to/language/root',
        ...  'title': 'English',
        ...  'code': 'en'}
        """

    def get_related_languages(self):
        """Returns a list of dicts of all related languages active in this
        area but not including itself.

        Example language dict:

        >>> {'url': 'http://url/to/language/root',
        ...  'title': 'English',
        ...  'code': 'en'}
        """



from ftw.subsite.negotiator import get_subsite_language


def set_subsite_language(event):
    """Force setting the language if traversing on a subsite.
    This automatically overrides everything defined in the language tool."""

    subsite_language = get_subsite_language(event.request)
    if subsite_language:
        event.request['LANGUAGE'] = subsite_language
    else:
        return



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.subsite.tests.helpers import introduce_language_subsites
from ftw.subsite.tests.pages import LanguageSwitcher
from ftw.testbrowser import browsing
from unittest import TestCase


class TestLanguageSwitcher(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    @browsing
    def test_shows_other_referenced_languages(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite FR')
                        .with_language('fr'))
        italian = create(Builder('subsite')
                         .titled(u'Subsite IT')
                         .with_language('it'))
        introduce_language_subsites(german, french, italian)

        browser.login().visit(german)
        self.assertEquals([u'Fran\xe7ais', u'Italiano'],
                          LanguageSwitcher().languages)

    @browsing
    def test_going_to_another_language(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite FR')
                        .with_language('fr'))
        introduce_language_subsites(german, french)

        browser.login().visit(german)
        LanguageSwitcher().click_language(u'Fran\xe7ais')
        self.assertEquals(french.absolute_url(), browser.url)
        LanguageSwitcher().click_language(u'Deutsch')
        self.assertEquals(german.absolute_url(), browser.url)

    @browsing
    def test_switch_is_invisible_unless_languages_hooked_up(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite FR')
                        .with_language('fr'))

        browser.login().visit(german)
        self.assertFalse(LanguageSwitcher().available)
        introduce_language_subsites(german, french)
        browser.login().visit(german)
        self.assertTrue(LanguageSwitcher().available)

    @browsing
    def test_other_site_not_visible_when_no_subsite_language_defined(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        unkown = create(Builder('subsite')
                        .titled(u'Subsite ...'))
        introduce_language_subsites(german, unkown)

        browser.login().visit(german)
        self.assertEquals([], LanguageSwitcher().languages)

    @browsing
    def test_does_not_show_subsites_of_other_language_groups(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite FR')
                        .with_language('fr'))
        italian = create(Builder('subsite')
                         .titled(u'Subsite IT')
                         .with_language('it'))
        spanish = create(Builder('subsite')
                         .titled(u'Subsite ES')
                         .with_language('es'))

        introduce_language_subsites(german, french)
        introduce_language_subsites(italian, spanish)

        browser.login()

        self.assertEquals(
            {'german': [u'Fran\xe7ais'],
             'french': [u'Deutsch'],
             'italian': [u'Espa\xf1ol'],
             'spanish': [u'Italiano']},

            {'german': LanguageSwitcher().visit(german).languages,
             'french': LanguageSwitcher().visit(french).languages,
             'italian': LanguageSwitcher().visit(italian).languages,
             'spanish': LanguageSwitcher().visit(spanish).languages})

    @browsing
    def test_hooking_up_language_subsites_with_site_root(self, browser):
        # Assumed that the site root is configured to be english.
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de')
                        .having(link_site_in_languagechooser=True))
        french = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('fr')
                        .having(link_site_in_languagechooser=True))

        introduce_language_subsites(german, french)

        browser.login()

        self.assertEquals(
            {'site root': [u'Deutsch', u'Fran\xe7ais'],
             'german': [u'English', u'Fran\xe7ais'],
             'french': [u'Deutsch', u'English']},

            {'site root': LanguageSwitcher().visit_portal().languages,
             'german': LanguageSwitcher().visit(german).languages,
             'french': LanguageSwitcher().visit(french).languages})

    @browsing
    def test_listing_only_site_root_works(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de')
                        .having(link_site_in_languagechooser=True))

        browser.login().visit(german)
        self.assertEquals([u'English'], LanguageSwitcher().languages)

    @browsing
    def test_current_link_is_current_language(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite FR')
                        .with_language('fr'))
        introduce_language_subsites(german, french)

        browser.login().visit(german)
        self.assertEquals(u'Deutsch', LanguageSwitcher().current)

    @browsing
    def test_current_link_is_language_on_plone_site_root(self, browser):
        create(Builder('subsite')
               .titled(u'Subsite')
               .with_language('de')
               .having(link_site_in_languagechooser=True))

        browser.login().visit()
        self.assertEquals(u'English', LanguageSwitcher().current)



from ftw.testbrowser import browser


class LanguageSwitcher():

    @property
    def available(self):
        return bool(browser.css('#portal-languageselector'))

    @property
    def current(self):
        return browser.css(
            '#portal-languageselector .actionMenuHeader a').first.text

    @property
    def language_links(self):
        return browser.css('#portal-languageselector .actionMenuContent a')

    @property
    def languages(self, sort=False):
        return self.language_links.text

    def click_language(self, label):
        browser.find_link_by_text(label).click()
        return self

    def visit(self, obj):
        browser.visit(obj)
        return self

    def visit_portal(self):
        browser.visit()
        return self



from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def uninstall_plone5(portal):
    clean_plone5_registry(portal)


def clean_plone5_registry(portal):
    registry = getUtility(IRegistry)

    allowed_sizes = registry['plone.allowed_sizes']
    allowed_sizes.remove(u'bannerimage 960:130')
    allowed_sizes.remove(u'logo 215:56')
    registry['plone.allowed_sizes'] = allowed_sizes

    displayed_types = list(registry['plone.displayed_types'])
    displayed_types.remove(u'ftw.subsite.Subsite')
    registry['plone.displayed_types'] = tuple(displayed_types)



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.subsite.viewlets.cssviewlet import CSSViewlet
from ftw.testbrowser import browsing
from plone.app.testing import login
from unittest import TestCase
from zope.publisher.browser import BrowserView


class TestSubsite(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        self.subsite = create(Builder('subsite')
                              .titled(u'Subsute')
                              .having(additional_css='p {color: red;}'))

        self.styles = '<style type="text/css" media="all">p {color: red;}</style>'

    @browsing
    def test_cssviewlet_in_subsite(self, browser):
        browser.login().visit(self.subsite)

        self.assertIn(self.styles,
                      browser.contents,
                      'Did not found the substes styles.')
        browser.visit(self.portal)
        self.assertNotIn(self.styles,
                         browser.contents,
                         'Found unexpeted subsite styles.')

    @browsing
    def test_cssviewlet_in_subfolder(self, browser):
        folder = create(Builder('folder').within(self.subsite))
        browser.login().visit(folder)
        self.assertIn(self.styles,
                      browser.contents,
                      'Did not found the substes styles.')

    def test_do_not_fail_if_nav_root_is_not_traversable(self):
        self.portal.portal_workflow.setChainForPortalTypes(
            ('Folder', 'ftw.subsite.Subsite'),
            'simple_publication_workflow')

        subsite = create(Builder('subsite').titled(u'MySubsite'))
        subfolder = create(Builder('folder').within(subsite))
        user = create(Builder('user').with_roles('Manager', on=subfolder))

        login(self.portal, user.getId())
        viewlet = CSSViewlet(subfolder,
                             subfolder.REQUEST,
                             BrowserView(subfolder, subfolder.REQUEST))

        self.assertEquals('', viewlet.render())



from zope.schema.interfaces import IVocabularyFactory
from zope.component import getUtility
from zope.schema.vocabulary import SimpleVocabulary
from ftw.subsite import _


class AvailableLanguagesVocabulary(object):
    """ Vocabulary base on plone.app.vocabularies.AvailableContentLanguages
    but also adds an empty term
    """

    def __call__(self, context):
        factory = getUtility(
            IVocabularyFactory,
            name='plone.app.vocabularies.AvailableContentLanguages',
            context=context)
        vocab = factory(context)
        terms = vocab._terms
        terms.append(vocab.createTerm("", "default", _(u"inherit_language", default=u"inherit language")))
        return SimpleVocabulary(terms)


AvailableLanguagesVocabularyFactory = AvailableLanguagesVocabulary()



from ftw.subsite.testing import FTW_SUBSITE_INTEGRATION_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from unittest import TestCase


class IntegrationTestCase(TestCase):

    layer = FTW_SUBSITE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])



from ftw.upgrade import UpgradeStep
import pkg_resources


IS_PLONE_5 = pkg_resources.get_distribution('Products.CMFPlone').version >= '5'


class RenameRessourcesFolderToResources(UpgradeStep):
    """Rename ressources folder to resources.
    """

    def __call__(self):
        if IS_PLONE_5:
            self.install_upgrade_profile(steps=['plone.app.registry'])
        else:
            self.install_upgrade_profile(steps=['cssregistry'])



"""Definition of the OrgUnit content type
"""

from ftw.subsite import _
from ftw.subsite.config import PROJECTNAME
from ftw.subsite.interfaces import ISubsite
from plone.app.blob.field import ImageField
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from zope.interface import implements


schema = atapi.Schema((
    ImageField(
        name='logo',
        required=False,
        storage=atapi.AnnotationStorage(),
        schemata='subsite',
        widget=atapi.ImageWidget(
            label=_(u'label_logo',
                    default=u'Logo'),
            description=_(u'help_logo',
                          default=u''))),

    atapi.TextField(
            name='additional_css',
            storage=atapi.AnnotationStorage(),
            schemata='subsite',
            allowable_content_types=('text/plain', ),
            default_content_type='text/plain',
            default_input_type='text/plain',
            default_output_type='text/plain',
            widget=atapi.TextAreaWidget(
                rows=15,
                label=_(u'label_additional_css',
                        default=u'Additional CSS'),
                description=_(u'help_additional_css',
                              default=u''))),

    atapi.ReferenceField(
        name='language_references',
        storage=atapi.AnnotationStorage(),
        schemata='subsite',
        multiValued=True,
        allowed_types=['Subsite'],
        relationship='subsite_subsite',
        widget=atapi.ReferenceWidget(
            checkbox_bound=1,
            label=_(u'label_language_references',
                    default=u'Languages'),
            description=_(u'help_language_references',
                          default=_(u'The language switch will only be '
                                     'displayed, if the chosen Subsite(s)'
                                     'has a value in the forcelangage '
                                     'field')))),

    atapi.BooleanField(
            name='linkSiteInLanguagechooser',
            accessor='showLinkToSiteInLanguageChooser',
            schemata='subsite',
            default=False,
            widget=atapi.BooleanWidget(
                label=_(u'Link Plone Site in language chooser'))),

    atapi.StringField(
        name="forcelanguage",
        default="",
        vocabulary_factory="ftw.subsites.languages",
        schemata='subsite',
        widget=atapi.SelectionWidget(
            label=_(u'label_forcelanguage', default=u'Subsite language'),
            description=_(u'help_forcelanguage',
                          default=u'The Subsite and it\'s content will be '
                                   'delivered in the choosen language'))),

    atapi.StringField(
        name="FromName",
        widget=atapi.StringWidget(
            label=_(u'label_fromname', default=u'Email Sendername'),
            description=_(u'help_fromname', default=u''))),

    atapi.StringField(
        name="FromEmail",
        widget=atapi.StringWidget(
            label=_(u'label_fromemail', default=u'Email Senderaddress'),
            description=_(u'help_fromname', default=u'')))
))


subsite_schema = folder.ATFolder.schema.copy() + schema

schemata.finalizeATCTSchema(
    subsite_schema,
    folderish=True,
    moveDiscussion=False,
    )


class Subsite(folder.ATFolder):
    """Subsite Unit"""
    implements(ISubsite, INavigationRoot)

    schema = subsite_schema
    meta_type = "Subsite"

atapi.registerType(Subsite, PROJECTNAME)
